/*
 * Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
 * Copyright 2006 Lorenzo Keller <lorenzo.keller@gmail.com>
 * 
 *     This file is part of Teacher.
 *
 *   Teacher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Teacher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Teacher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package kello.teacher.rfb;

public class RfbOptions {

	//
	// The actual data which other classes look at:
	//

	private int[] encodings = new int[10];

	private int nEncodings;

	private boolean reverseMouseButtons2And3;

	private boolean drawEachPixelForRawRects;

	private boolean copyRectFast;

	private boolean shareDesktop;

	private boolean useCopyRect;

	private int defaultEncoding;

	public RfbOptions() {
		super();

		// Set up defaults

		defaultEncoding = rfbProto.EncodingHextile;
		useCopyRect = true;
		reverseMouseButtons2And3 = false;
		drawEachPixelForRawRects = true;
		copyRectFast = true;
		shareDesktop = true;

		setDefaultEncodings();
	}

	private void setDefaultEncodings() {
		nEncodings = 0;
		if (useCopyRect) {
			encodings[nEncodings++] = rfbProto.EncodingCopyRect;
		}

		int preferredEncoding = rfbProto.EncodingRaw;

		preferredEncoding = defaultEncoding;

		if (preferredEncoding == rfbProto.EncodingRaw) {
			drawEachPixelForRawRects = false;
		}

		encodings[nEncodings++] = preferredEncoding;
		if (preferredEncoding != rfbProto.EncodingRRE) {
			encodings[nEncodings++] = rfbProto.EncodingRRE;
		}
		if (preferredEncoding != rfbProto.EncodingCoRRE) {
			encodings[nEncodings++] = rfbProto.EncodingCoRRE;
		}
		if (preferredEncoding != rfbProto.EncodingHextile) {
			encodings[nEncodings++] = rfbProto.EncodingHextile;
		}
	}

	public void addEncoding( int value) {
		encodings[nEncodings++] = value;
	}

	public int[] getEncodings() {
		return (int[]) encodings.clone();
	}

	public boolean isShareDesktop() {
		return shareDesktop;
	}

	public void setShareDesktop(boolean shareDesktop) {
		this.shareDesktop = shareDesktop;
	}

	public boolean isCopyRectFast() {
		return copyRectFast;
	}

	public void setCopyRectFast(boolean copyRectFast) {
		this.copyRectFast = copyRectFast;
	}

	public boolean isDrawEachPixelForRawRects() {
		return drawEachPixelForRawRects;
	}

	public void setDrawEachPixelForRawRects(boolean drawEachPixelForRawRects) {
		this.drawEachPixelForRawRects = drawEachPixelForRawRects;
	}

	public boolean isReverseMouseButtons2And3() {
		return reverseMouseButtons2And3;
	}

	public void setReverseMouseButtons2And3(boolean reverseMouseButtons2And3) {
		this.reverseMouseButtons2And3 = reverseMouseButtons2And3;
	}

	public boolean isUseCopyRect() {
		return useCopyRect;
	}

	public void setUseCopyRect(boolean useCopyRect) {
		this.useCopyRect = useCopyRect;
	}

	public int getNEncodings() {
		return nEncodings;
	}

}
