/*
 * Copyright 2006 Lorenzo Keller <lorenzo.keller@gmail.com>
 * 
 *     This file is part of Teacher.
 *
 *   Teacher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Teacher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Teacher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package kello.teacher.rfb.viewer;

import java.awt.BorderLayout;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

public class JRfbFrame extends JFrame {

	private JPanel jContentPane = null;
	private JScrollPane jScrollPane = null;
	private JRfbViewer jRfbViewer = null;
	private boolean scaled;
	private boolean fullscreen;

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJRfbViewer());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jRfbViewer	
	 * 	
	 * @return kello.teacher.rfb.viewer.JRfbViewer	
	 */
	private JRfbViewer getJRfbViewer() {
		if (jRfbViewer == null) {
			jRfbViewer = new JRfbViewer(scaled);
			
		}
		return jRfbViewer;
	}

	/**
	 * 
	 * Use this to  test the frame
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		JRfbFrame frame = new JRfbFrame(false,true);
		frame.setVisible(true);
		frame.setHost("127.0.0.1");
		frame.setPort(5901);
		frame.setPassword("lorenzo_40");
		frame.setSendEvents(true);
		frame.start();
	}

	/**
	 * This is the default constructor
	 * 
	 * @param scaled the framebuffer should be scaled to the frame size
	 * @param fullscreen the frame should be fullscreen
	 */
	public JRfbFrame(boolean scaled, boolean fullscreen) {
		super();
		this.scaled = scaled;
		this.fullscreen = fullscreen;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setContentPane(getJContentPane());
		this.setTitle("Remote Screen");
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				if (jRfbViewer != null) jRfbViewer.stop();
				dispose();
			}			
		});
		
		
		
		if (fullscreen) {
			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			
			GraphicsDevice gd = ge.getDefaultScreenDevice();
			this.setUndecorated(true);
			gd.setFullScreenWindow(this);
		}
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			if (!scaled) {
				jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			} else {
				jContentPane.add(getJRfbViewer(), java.awt.BorderLayout.CENTER);
			}
		}
		return jContentPane;
	}

	public String getHost() {
		return jRfbViewer.getHost();
	}

	public int getPort() {
		return jRfbViewer.getPort();
	}

	public boolean isSendingEvents() {
		return jRfbViewer.isSendingEvents();
	}

	public void setHost(String host) {
		setTitle("Remote Screen @ " + host);
		jRfbViewer.setHost(host);
	}

	public void setPassword(String password) {
		jRfbViewer.setPassword(password);
	}

	public void setPort(int port) {
		jRfbViewer.setPort(port);
	}

	public void setSendEvents(boolean send) {
		jRfbViewer.setSendEvents(send);
	}

	public void start() {
		jRfbViewer.start();
	}

	public void stop() {
		jRfbViewer.stop();
	}		
	
}
