/*
 * Copyright 2006 Lorenzo Keller <lorenzo.keller@gmail.com>
 * 
 *     This file is part of Teacher.
 *
 *   Teacher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Teacher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Teacher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package kello.teacher.teacher;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import kello.teacher.protocol.StudentAnnounceMessage;
import kello.teacher.rfb.viewer.JRfbViewer;

public class JStudentThumbnail extends JInternalFrame {

	private JRfbViewer vncviewer3 = null;
	
	private Timer timer;
	
	private StudentHandler student;

	private JPopupMenu jPopupMenu = null;

	private JCheckBoxMenuItem jLocked = null;

	private JCheckBoxMenuItem jDemo = null;


	/**
	 * This is the default constructor
	 */
	public JStudentThumbnail(StudentHandler student) {
		super();
		initialize();
		
		this.student = student;
		vncviewer3.setHost(student.getHost());
		vncviewer3.setPassword(student.getHub().getTeacher().getProperty("vnc-client-password"));
		timer = new Timer();
		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(340, 240);
		this.setResizable(true);
		this.setMaximizable(true);
		this.setIconifiable(true);
		setVisible(true);
		setContentPane(getVncviewer3());		
	}

	/**
	 * This method initializes vncviewer3	
	 * 	
	 * @return kello.teacher.demoviewer.vncviewer	
	 */
	private JRfbViewer getVncviewer3() {
		if (vncviewer3 == null) {
			vncviewer3 = new JRfbViewer();
			//vncviewer3.setSize(jPanel.getSize());
			vncviewer3.addMouseListener(new java.awt.event.MouseAdapter() {   
				public void mouseClicked(java.awt.event.MouseEvent e) {    
					if (e.getClickCount() == 2) {
						student.startVNCviewer();
					}
				}
				public void mousePressed(java.awt.event.MouseEvent e) {
					if (e.isPopupTrigger()) {						
						getJPopupMenu().show(e.getComponent(), e.getX(), e.getY());
		            }
				}
			       public void mouseReleased(java.awt.event.MouseEvent e) {
			            if (e.isPopupTrigger()) {
			                getJPopupMenu().show(e.getComponent(), e.getX(), e.getY());
			            }
			        }
				
			});
			//vncviewer3.setSize(200,200);
		}
		return vncviewer3;
	}
	
	
	public void showScreen () {
		
		TimerTask task = new TimerTask(){
			public void run () {
				if (vncviewer3.isStopped())					
					vncviewer3.start();			
			}
		};

		timer.schedule(task,0,Integer.parseInt(student.getHub().getTeacher().getProperty("vnc-client-reconnect-timeout")));
	}
	
	public void hideScreen() {
		timer.cancel();
		vncviewer3.stop();
	}
	
	public void dispose() {
		hideScreen();
		super.dispose();
	}
	
	public void notifyNewAnnouncement() {
		
		StudentAnnounceMessage amsg = student.getLastAnnouncement();

		String name = student.getHub().getTeacher().getFullname(amsg.getName());
		if (name == null || name.equals("")) {
			try {
				name = InetAddress.getByName(student.getHost()).getHostName();
			} catch (UnknownHostException ex) {
				name = student.getHost();
			}
		}
		
		this.setTitle(name);
		
		getJLocked().setSelected(amsg.isLocked());
		getJDemo().setSelected(amsg.isInDemo());
		
		if (amsg.isFullscreen() && amsg.isInDemo()) {
			vncviewer3.setPaused(true);
		} else {
			vncviewer3.setPaused(false);
		}
		
		vncviewer3.setPort(amsg.getVncport());
		if (vncviewer3.isStopped())					
			vncviewer3.start();		
		
	}

	/**
	 * This method initializes jPopupMenu	
	 * 	
	 * @return javax.swing.JPopupMenu	
	 */
	private JPopupMenu getJPopupMenu() {
		if (jPopupMenu == null) {
			jPopupMenu = new JPopupMenu();
			jPopupMenu.add(getJLocked());
			jPopupMenu.add(getJDemo());
			
			ArrayList cmds = student.getHub().getTeacher().getAllCommands();
			
			Iterator i = cmds.iterator();
			

			
			while (i.hasNext()) {
				final Command cmd = (Command) i.next();

				ActionListener a =  new ActionListener()  {

					public void actionPerformed(ActionEvent e) {
						student.executeCommand(cmd);
					}
					
				};
				
				JMenuItem item = new JMenuItem(cmd.getDescription());
				item.addActionListener(a);
				jPopupMenu.add(item);
				
			}
			
		}
		return jPopupMenu;
	}

	/**
	 * This method initializes jCheckBoxMenuItem	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */
	private JCheckBoxMenuItem getJLocked() {
		if (jLocked == null) {
			jLocked = new JCheckBoxMenuItem();
			jLocked.setText("Locked");
			jLocked.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					student.sendLockScreen(jLocked.isSelected());
				}
			});
		}
		return jLocked;
	}

	/**
	 * This method initializes jCheckBoxMenuItem	
	 * 	
	 * @return javax.swing.JCheckBoxMenuItem	
	 */
	private JCheckBoxMenuItem getJDemo() {
		if (jDemo == null) {
			jDemo = new JCheckBoxMenuItem();
			jDemo.setText("Demo");
			jDemo.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (jDemo.isSelected()) {
						JTeacherFrame teacher = student.getHub().getTeacher();
						student.sendDemoStart(student.getHub().getDesc(), student.getHub().getMulticastGroup(), student.getHub().getMulticastPort(),teacher.isFullscreenSelected());
					} else {
						student.sendDemoStop();
					}
				}
			});
		}
		return jDemo;
	}


	public StudentHandler getStudent() {
		return student;
	}


}
