/*
 * Copyright 2006 Lorenzo Keller <lorenzo.keller@gmail.com>
 * 
 *     This file is part of Teacher.
 *
 *   Teacher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Teacher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Teacher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package kello.teacher.teacher;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class JTeacherConfigurationFrame extends JDialog {

	private JPanel jContentPane = null;

	private JTabbedPane jTabbedPane = null;

	private JPanel jPanel = null;

	private JPanel jPanel1 = null;

	private JLabel jLabel = null;

	private JTextField serverHost = null;

	private JLabel jLabel1 = null;

	private JTextField serverPort = null;

	private JPasswordField serverPassword = null;

	private JLabel jLabel2 = null;

	private JPasswordField clientPassword = null;

	private JPanel jPanel2 = null;

	private JButton save = null;

	private JButton close = null;

	private JLabel jLabel4 = null;

	private JTextField screengroup = null;

	private JLabel jLabel6 = null;

	private JLabel jLabel7 = null;

	private JTextField screenPort = null;

	private JTextField screenTTL = null;

	private JLabel jLabel5 = null;

	private JLabel jLabel8 = null;

	private JTextField announceGroup = null;

	private JLabel jLabel9 = null;

	private JTextField announcePort = null;

	private JLabel jLabel10 = null;

	private JTextField announceTTL = null;

	private JLabel jLabel11 = null;

	private JTextField announceInterval = null;

	private JPanel jPanel3 = null;

	private JLabel jLabel12 = null;

	private JTextField teacherPort = null;

	private JLabel jLabel13 = null;

	private JLabel jLabel14 = null;

	private JTextField clientReconnectTimeout = null;

	private JPanel jPanel4 = null;

	private JLabel jLabel16 = null;

	private JTextField fingerCommand = null;

	private JLabel jLabel17 = null;

	private JScrollPane jScrollPane = null;

	private JTable commands = null;

	private JTeacherFrame teacher;

	private JButton revert = null;

	/**
	 * This is the default constructor
	 */
	public JTeacherConfigurationFrame(JTeacherFrame teacher) {
		super();
		this.teacher = teacher;
		initialize();
		loadData();
	}

	private void loadData() {

		serverHost.setText(teacher.getProperty("vncserver"));
		serverPort.setText(teacher.getProperty("vncport"));
		serverPassword.setText(teacher.getProperty("vncpassword"));
		screengroup.setText(teacher.getProperty("screen-multicast-group"));
		screenPort.setText(teacher.getProperty("screen-multicast-port"));
		screenTTL.setText(teacher.getProperty("screen-multicast-ttl"));
		announceGroup.setText(teacher.getProperty("teacher-multicast-group"));
		announcePort.setText(teacher.getProperty("teacher-multicast-port"));
		announceTTL.setText(teacher.getProperty("teacher-multicast-ttl"));
		announceInterval.setText(teacher
				.getProperty("teacher-multicast-interval"));
		teacherPort.setText(teacher.getProperty("teacher-port"));
		clientReconnectTimeout.setText(teacher
				.getProperty("vnc-client-reconnect-timeout"));
		clientPassword.setText(teacher.getProperty("vnc-client-password"));
		fingerCommand.setText(teacher.getProperty("finger"));

		DefaultTableModel model = new DefaultTableModel();

		model.addColumn("Name");
		model.addColumn("Description");
		model.addColumn("command");

		Iterator i = teacher.getAllCommands().iterator();

		while (i.hasNext()) {
			Command cmd = (Command) i.next();
			Vector row = new Vector();
			row.add(cmd.getName());
			row.add(cmd.getDescription());
			row.add(cmd.getCommand());
			model.addRow(row);
		}

		commands.setModel(model);

	}

	private void saveData() throws NumberFormatException {

		/* check that all numeric values are valid,
		 * this will raise an NumberFormatException
		 * that has to be intercepted by the caller */
		
		Integer.parseInt(serverPort.getText());
		Integer.parseInt(serverPort.getText());
		Integer.parseInt(screenTTL.getText());
		Integer.parseInt(announcePort.getText());
		Integer.parseInt(announceTTL.getText());
		Integer.parseInt(announceInterval.getText());
		Integer.parseInt(teacherPort.getText());
		Integer.parseInt(clientReconnectTimeout.getText());


		
		teacher.setProperty("vncserver",serverHost.getText());
		teacher.setProperty("vncport",serverPort.getText());
		teacher.setProperty("vncpassword",new String(serverPassword.getPassword()));
		teacher.setProperty("screen-multicast-group",screengroup.getText());
		teacher.setProperty("screen-multicast-port",serverPort.getText());
		teacher.setProperty("screen-multicast-ttl",screenTTL.getText());
		teacher.setProperty("teacher-multicast-group",announceGroup.getText());
		teacher.setProperty("teacher-multicast-port",announcePort.getText());
		teacher.setProperty("teacher-multicast-ttl",announceTTL.getText());
		teacher.setProperty("teacher-multicast-interval",announceInterval.getText());
		teacher.setProperty("teacher-port",teacherPort.getText());
		teacher.setProperty("vnc-client-reconnect-timeout",clientReconnectTimeout.getText());
		teacher.setProperty("vnc-client-password",new String(clientPassword.getPassword()));
		teacher.setProperty("finger",fingerCommand.getText());

		
		ArrayList commandsArray = teacher.getAllCommands();
	
		commandsArray.clear();
		
		Iterator i = ((DefaultTableModel) commands.getModel()).getDataVector().iterator();

		while (i.hasNext()) {
			Vector row =  (Vector) i.next();
			Command cmd = new Command((String) row.elementAt(0), (String) row.elementAt(1), (String) row.elementAt(2));
			commandsArray.add(cmd);

		}

	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(744, 589);
		this.setContentPane(getJContentPane());
		this.setTitle("Teacher Configuration");
		this.setModal(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJTabbedPane(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTabbedPane
	 * 
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.addTab("VNC configurations", null, getJPanel(), null);
			jTabbedPane.addTab("Programs", null, getJPanel4(), null);
			jTabbedPane.addTab("Advanced", null, getJPanel3(), null);
		}
		return jTabbedPane;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridy = 0;
			gridBagConstraints3.ipady = 2;
			gridBagConstraints3.ipadx = 2;
			gridBagConstraints3.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints3.gridx = 2;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 2;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridy = 3;
			gridBagConstraints5.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints5.ipadx = 2;
			gridBagConstraints5.ipady = 2;
			gridBagConstraints5.gridwidth = 2;
			gridBagConstraints5.gridx = 1;
			GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
			gridBagConstraints21.gridx = 0;
			gridBagConstraints21.gridy = 3;
			jLabel2 = new JLabel();
			jLabel2.setText("Client password:");
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridy = 2;
			gridBagConstraints11.ipadx = 0;
			gridBagConstraints11.ipady = 0;
			gridBagConstraints11.weightx = 0.0;
			gridBagConstraints11.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints11.gridwidth = 2;
			gridBagConstraints11.gridx = 1;
			jLabel1 = new JLabel();
			jLabel1.setText("Teacher password:");
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.ipady = 2;
			gridBagConstraints1.ipadx = 2;
			gridBagConstraints1.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints1.gridx = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			jLabel = new JLabel();
			jLabel.setText("Teacher screen: ");
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.add(jLabel, gridBagConstraints);
			jPanel.add(getServerHost(), gridBagConstraints1);
			jPanel.add(jLabel1, gridBagConstraints2);
			jPanel.add(getServerPort(), gridBagConstraints3);
			jPanel.add(getTeacherPassword(), gridBagConstraints11);
			jPanel.add(jLabel2, gridBagConstraints21);
			jPanel.add(getClientPassword(), gridBagConstraints5);
		}
		return jPanel;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			GridBagConstraints gridBagConstraints20 = new GridBagConstraints();
			gridBagConstraints20.gridx = 0;
			gridBagConstraints20.gridwidth = 5;
			gridBagConstraints20.anchor = java.awt.GridBagConstraints.WEST;
			gridBagConstraints20.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints20.gridy = 2;
			jLabel11 = new JLabel();
			jLabel11
					.setText("Configure here the multicast address where the teacher will announce itself");
			GridBagConstraints gridBagConstraints19 = new GridBagConstraints();
			gridBagConstraints19.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints19.gridy = 3;
			gridBagConstraints19.weightx = 1.0;
			gridBagConstraints19.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints19.gridx = 5;
			GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
			gridBagConstraints18.gridx = 4;
			gridBagConstraints18.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints18.gridy = 3;
			jLabel10 = new JLabel();
			jLabel10.setText("TTL");
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints17.gridy = 3;
			gridBagConstraints17.weightx = 1.0;
			gridBagConstraints17.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints17.gridx = 3;
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.gridx = 2;
			gridBagConstraints16.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints16.gridy = 3;
			jLabel9 = new JLabel();
			jLabel9.setText("Port");
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints15.gridy = 3;
			gridBagConstraints15.weightx = 1.0;
			gridBagConstraints15.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints15.gridx = 1;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.gridx = 0;
			gridBagConstraints14.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints14.gridy = 3;
			jLabel8 = new JLabel();
			jLabel8.setText("Group:");
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.gridwidth = 5;
			gridBagConstraints13.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints13.anchor = java.awt.GridBagConstraints.WEST;
			gridBagConstraints13.gridy = 0;
			jLabel5 = new JLabel();
			jLabel5
					.setText("Configure here the multicast address where the screen of the teacher will be sent");
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints12.gridy = 1;
			gridBagConstraints12.weightx = 1.0;
			gridBagConstraints12.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints12.gridx = 5;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints8.gridy = 1;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints8.gridx = 3;
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 4;
			gridBagConstraints10.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints10.gridy = 1;
			jLabel7 = new JLabel();
			jLabel7.setText("TTL");
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 2;
			gridBagConstraints9.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints9.gridy = 1;
			jLabel6 = new JLabel();
			jLabel6.setText("Port");
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 1;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints7.gridwidth = 1;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints4.gridy = 1;
			jLabel4 = new JLabel();
			jLabel4.setText("Group:");
			jPanel1 = new JPanel();
			jPanel1.setLayout(new GridBagLayout());
			jPanel1.add(jLabel4, gridBagConstraints4);
			jPanel1.add(getScreengroup(), gridBagConstraints7);
			jPanel1.add(jLabel6, gridBagConstraints9);
			jPanel1.add(jLabel7, gridBagConstraints10);
			jPanel1.add(getScreenPort(), gridBagConstraints8);
			jPanel1.add(getScreenTTL(), gridBagConstraints12);
			jPanel1.add(jLabel5, gridBagConstraints13);
			jPanel1.add(jLabel8, gridBagConstraints14);
			jPanel1.add(getAnnounceGroup(), gridBagConstraints15);
			jPanel1.add(jLabel9, gridBagConstraints16);
			jPanel1.add(getAnnouncePort(), gridBagConstraints17);
			jPanel1.add(jLabel10, gridBagConstraints18);
			jPanel1.add(getAnnounceTTL(), gridBagConstraints19);
			jPanel1.add(jLabel11, gridBagConstraints20);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getServerHost() {
		if (serverHost == null) {
			serverHost = new JTextField();
			serverHost.setColumns(15);
		}
		return serverHost;
	}

	/**
	 * This method initializes jTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getServerPort() {
		if (serverPort == null) {
			serverPort = new JTextField();
			serverPort.setColumns(5);
		}
		return serverPort;
	}

	/**
	 * This method initializes jTextField2
	 * 
	 * @return javax.swing.JTextField
	 */
	private JPasswordField getTeacherPassword() {
		if (serverPassword == null) {
			serverPassword = new JPasswordField();
			serverPassword.setColumns(20);
		}
		return serverPassword;
	}

	/**
	 * This method initializes jTextField3
	 * 
	 * @return javax.swing.JTextField
	 */
	private JPasswordField getClientPassword() {
		if (clientPassword == null) {
			clientPassword = new JPasswordField();
			clientPassword.setColumns(20);
		}
		return clientPassword;
	}

	/**
	 * This method initializes jPanel2
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.add(getSave(), null);
			jPanel2.add(getClose(), null);
			jPanel2.add(getRevert(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getSave() {
		if (save == null) {
			save = new JButton();
			save.setText("Save and close");
			save.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
				try {
					saveData();
					teacher.saveConfiguration();
					JOptionPane.showMessageDialog(JTeacherConfigurationFrame.this,"Changes will take effect at next restart");
					JTeacherConfigurationFrame.this.dispose();
				} catch (NumberFormatException ex) {
					JOptionPane.showMessageDialog(JTeacherConfigurationFrame.this, "Some value inserted are not valid, aborting saving changes","Error",JOptionPane.ERROR_MESSAGE);			
				}
				}
			});
		}
		return save;
	}

	/**
	 * This method initializes jButton1
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getClose() {
		if (close == null) {
			close = new JButton();
			close.setText("Close without saving");
			close.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					JTeacherConfigurationFrame.this.dispose();
				}
			});
		}
		return close;
	}

	/**
	 * This method initializes jTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getScreengroup() {
		if (screengroup == null) {
			screengroup = new JTextField();
		}
		return screengroup;
	}

	/**
	 * This method initializes jTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getScreenPort() {
		if (screenPort == null) {
			screenPort = new JTextField();
			screenPort.setColumns(5);
		}
		return screenPort;
	}

	/**
	 * This method initializes jTextField3
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getScreenTTL() {
		if (screenTTL == null) {
			screenTTL = new JTextField();
			screenTTL.setColumns(5);
		}
		return screenTTL;
	}

	/**
	 * This method initializes jTextField4
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAnnounceGroup() {
		if (announceGroup == null) {
			announceGroup = new JTextField();
		}
		return announceGroup;
	}

	/**
	 * This method initializes jTextField5
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAnnouncePort() {
		if (announcePort == null) {
			announcePort = new JTextField();
			announcePort.setColumns(5);
		}
		return announcePort;
	}

	/**
	 * This method initializes jTextField6
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAnnounceTTL() {
		if (announceTTL == null) {
			announceTTL = new JTextField();
			announceTTL.setColumns(5);
		}
		return announceTTL;
	}

	/**
	 * This method initializes jTextField7
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAnnounceInterval() {
		if (announceInterval == null) {
			announceInterval = new JTextField();
		}
		return announceInterval;
	}

	/**
	 * This method initializes jPanel3
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			GridBagConstraints gridBagConstraints28 = new GridBagConstraints();
			gridBagConstraints28.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints28.gridy = 3;
			gridBagConstraints28.weightx = 1.0;
			gridBagConstraints28.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints28.gridx = 1;
			GridBagConstraints gridBagConstraints27 = new GridBagConstraints();
			gridBagConstraints27.gridx = 0;
			gridBagConstraints27.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints27.gridy = 3;
			jLabel14 = new JLabel();
			jLabel14.setText("VNC client reconnect time");
			jLabel14.setToolTipText("");
			GridBagConstraints gridBagConstraints23 = new GridBagConstraints();
			gridBagConstraints23.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints23.gridy = 1;
			gridBagConstraints23.weightx = 1.0;
			gridBagConstraints23.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints23.gridx = 1;
			GridBagConstraints gridBagConstraints25 = new GridBagConstraints();
			gridBagConstraints25.gridx = 0;
			gridBagConstraints25.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints25.gridy = 2;
			GridBagConstraints gridBagConstraints24 = new GridBagConstraints();
			gridBagConstraints24.gridx = 0;
			gridBagConstraints24.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints24.gridy = 1;
			jLabel13 = new JLabel();
			jLabel13.setText("Teacher Unicast port");
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints22.gridx = 1;
			gridBagConstraints22.gridy = 2;
			gridBagConstraints22.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints22.weightx = 1.0;
			jLabel12 = new JLabel();
			jLabel12.setText("Teacher Announcement interval");
			GridBagConstraints gridBagConstraints26 = new GridBagConstraints();
			gridBagConstraints26.gridy = 0;
			gridBagConstraints26.gridwidth = 2;
			gridBagConstraints26.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints26.gridx = 0;
			jPanel3 = new JPanel();
			jPanel3.setLayout(new GridBagLayout());
			jPanel3.add(getJPanel1(), gridBagConstraints26);
			jPanel3.add(jLabel12, gridBagConstraints24);
			jPanel3.add(getTeacherPort(), gridBagConstraints22);
			jPanel3.add(jLabel13, gridBagConstraints25);
			jPanel3.add(getAnnounceInterval(), gridBagConstraints23);
			jPanel3.add(jLabel14, gridBagConstraints27);
			jPanel3.add(getClientReconnectTimeout(), gridBagConstraints28);
		}
		return jPanel3;
	}

	/**
	 * This method initializes jTextField8
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTeacherPort() {
		if (teacherPort == null) {
			teacherPort = new JTextField();
		}
		return teacherPort;
	}

	/**
	 * This method initializes jTextField9
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getClientReconnectTimeout() {
		if (clientReconnectTimeout == null) {
			clientReconnectTimeout = new JTextField();
		}
		return clientReconnectTimeout;
	}

	/**
	 * This method initializes jPanel4
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			GridBagConstraints gridBagConstraints35 = new GridBagConstraints();
			gridBagConstraints35.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints35.gridy = 3;
			gridBagConstraints35.weightx = 1.0;
			gridBagConstraints35.weighty = 1.0;
			gridBagConstraints35.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints35.gridx = 1;
			GridBagConstraints gridBagConstraints34 = new GridBagConstraints();
			gridBagConstraints34.gridx = 0;
			gridBagConstraints34.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints34.anchor = java.awt.GridBagConstraints.NORTHWEST;
			gridBagConstraints34.gridy = 3;
			jLabel17 = new JLabel();
			jLabel17.setText("Commands:");
			GridBagConstraints gridBagConstraints33 = new GridBagConstraints();
			gridBagConstraints33.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints33.gridy = 1;
			gridBagConstraints33.weightx = 1.0;
			gridBagConstraints33.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints33.gridx = 1;
			GridBagConstraints gridBagConstraints32 = new GridBagConstraints();
			gridBagConstraints32.gridx = 0;
			gridBagConstraints32.insets = new java.awt.Insets(12, 12, 12, 12);
			gridBagConstraints32.anchor = java.awt.GridBagConstraints.WEST;
			gridBagConstraints32.gridy = 1;
			jLabel16 = new JLabel();
			jLabel16.setText("Finger command:");
			jPanel4 = new JPanel();
			jPanel4.setLayout(new GridBagLayout());
			jPanel4.add(jLabel16, gridBagConstraints32);
			jPanel4.add(getFingerCommand(), gridBagConstraints33);
			jPanel4.add(jLabel17, gridBagConstraints34);
			jPanel4.add(getJScrollPane(), gridBagConstraints35);
		}
		return jPanel4;
	}

	/**
	 * This method initializes jTextField11
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getFingerCommand() {
		if (fingerCommand == null) {
			fingerCommand = new JTextField();
		}
		return fingerCommand;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getCommands());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getCommands() {
		if (commands == null) {

			commands = new JTable();
			commands.setCellEditor(new DefaultCellEditor(new JTextField()));

			
			/* creates a new popup menu for the table */
			final JPopupMenu popupMenu = new JPopupMenu();
			final JPopupMenu popupMenu2 = new JPopupMenu();
			
			final JMenuItem addRow = new JMenuItem("Add new command");
			final JMenuItem addRow2 = new JMenuItem("Add new command");
			final JMenuItem deleteRow = new JMenuItem("Delete selected command");

			// this action listener will handle actions on the menu items
			ActionListener menuActions = new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					if (e.getSource().equals(addRow) || e.getSource().equals(addRow2)) {
						((DefaultTableModel) commands.getModel()).addRow(new Vector());
					} else if (e.getSource().equals(deleteRow) && commands.getSelectedRow() != -1) {						
						((DefaultTableModel) commands.getModel()).removeRow(commands.getSelectedRow());
					}
					
				}
				
			};
			
			addRow.addActionListener(menuActions);
			addRow2.addActionListener(menuActions);
			deleteRow.addActionListener(menuActions);
						
			popupMenu.add(addRow);
			popupMenu.add(deleteRow);
			
			popupMenu2.add(addRow2);
			
			/*  show the popup menu when rightclick is done on the table */
			MouseListener mouseListener = new MouseAdapter() {
				public void selectAndPopup(MouseEvent e) {
					if (e.isPopupTrigger()) {
						/* select the row under the cursor */
						Point p = new Point(e.getX(), e.getY());
						int row = commands.rowAtPoint(p);
						if (row >= 0 && row < commands.getRowCount()) {
							commands.getSelectionModel().setSelectionInterval(row, row);
						}
						popupMenu.show(e.getComponent(), e.getX(), e.getY());
					}
				}
				public void mousePressed(MouseEvent e) {
					selectAndPopup(e);	
				}

				public void mouseReleased(MouseEvent e) {
					selectAndPopup(e);
				}

			};
			commands.addMouseListener(mouseListener);
			
			/*  show the popup menu when right click is done on an empty table*/
			MouseListener mouseListener2 = new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isPopupTrigger()) {
						popupMenu2.show(e.getComponent(), e.getX(), e.getY());
					}
				}

				public void mouseReleased(MouseEvent e) {
					if (e.isPopupTrigger()) {
						popupMenu2.show(e.getComponent(), e.getX(), e.getY());
					}
				}

			};
			jScrollPane.getViewport().addMouseListener(mouseListener2);
		}
		return commands;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getRevert() {
		if (revert == null) {
			revert = new JButton();
			revert.setText("Revert to old values");
			revert.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					loadData();
				}
			});
		}
		return revert;
	}

}
