/*
 * Copyright 2006 Lorenzo Keller <lorenzo.keller@gmail.com>
 * 
 *     This file is part of Teacher.
 *
 *   Teacher is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Teacher is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Teacher; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

package kello.teacher.teacher;

import java.awt.Frame;
import java.awt.Insets;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDesktopPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import kello.teacher.util.ExceptionDialogBox;
import kello.teacher.util.Utility;


public class JTeacherFrame extends Frame {

	private JToolBar buttonPanel = null;
	private JButton startDemoButton = null;
	private JButton stopDemoButton = null;
	private TeacherAnnouncementDaemon announcer;
	
	private StudentHub hub;

	private JCheckBox chkFullscreen = null;
	private JLabel lblFullscreen = null;
	private JToggleButton lockScreenButton = null;
	private JDesktopPane jDesktopPane = null;
	
	private HashMap fullnames;
	
	private Properties properties;
	private JButton saveConfigButton = null;
	private JButton ReloadLayoutButton = null;
	
	public JTeacherFrame () {
		super("Teacher control panel");
		initialize();
	}
	

	
	
	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		
        this.setTitle("Teacher Control Panel");
        
		this.fullnames = new HashMap();
		this.commands = new ArrayList();
        
        Properties defaultProperties = new Properties();
        
        defaultProperties.setProperty("vncserver", "127.0.0.1");	
        defaultProperties.setProperty("vncport", "5900");
    	defaultProperties.setProperty("vncpassword", "");
    	defaultProperties.setProperty("screen-multicast-group", "230.0.0.1");
    	defaultProperties.setProperty("screen-multicast-port", "3333");
    	defaultProperties.setProperty("screen-multicast-ttl", "16");
    	defaultProperties.setProperty("teacher-multicast-group", "230.0.0.2");
    	defaultProperties.setProperty("teacher-multicast-port", "3000");
    	defaultProperties.setProperty("teacher-multicast-ttl", "16");   
    	defaultProperties.setProperty("teacher-multicast-interval", "2000");
    	defaultProperties.setProperty("teacher-port", "3333");
    	defaultProperties.setProperty("height", "600");
    	defaultProperties.setProperty("width", "800");
    	defaultProperties.setProperty("vnc-client-reconnect-timeout", "10000");
    	defaultProperties.setProperty("vnc-client-password", "");
    	defaultProperties.setProperty("finger", "finger -l '%user'| head -1 | cut -f4 | cut -c 7-");        

        this.properties = new Properties(defaultProperties);
       
        if (!new File("teacher.properties").exists() || !new File("commands.properties").exists()) {
        	JOptionPane.showMessageDialog(this, "The configuration file was not found, please configure the application","Install",JOptionPane.INFORMATION_MESSAGE);
			JTeacherConfigurationFrame conf = new JTeacherConfigurationFrame(JTeacherFrame.this);
			conf.setVisible(true);        	
        }
        
        try {
        	this.properties.load(new FileInputStream("teacher.properties"));
      
		} catch (Exception ex) {
        	ExceptionDialogBox.displayExceptionDialog(this,ex, "Error while loading the configurations from teacher.properties (this is not fatal)");
        }

        this.add(getJPanel(), java.awt.BorderLayout.NORTH);
        this.add(getJDesktopPane(), java.awt.BorderLayout.CENTER);
	
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				System.exit(0);
			}
		});
		
        this.setSize(Integer.parseInt(properties.getProperty("width")),
        		Integer.parseInt(properties.getProperty("height")));
        

		try {
			hub = new StudentHub(Integer.parseInt(properties.getProperty("teacher-port")), this);
			hub.start();
		} catch (Exception ex) {
			ExceptionDialogBox.displayExceptionDialog(this,ex, "Fatal error while starting the teacher server");
		}
		
		try {
			announcer = new TeacherAnnouncementDaemon(properties.getProperty("teacher-multicast-group"),
													  Integer.parseInt(properties.getProperty("teacher-multicast-port")),
													  Integer.parseInt(properties.getProperty("teacher-multicast-ttl")),
													  Integer.parseInt(properties.getProperty("teacher-port")),
													  Integer.parseInt(properties.getProperty("teacher-multicast-interval")));
			announcer.start();
			
		} catch (Exception ex) {
			ExceptionDialogBox.displayExceptionDialog(this,ex, "Fatal error while connecting to control group");
		} 
		
		
        
	}
	
	public void saveConfiguration() {
		
		/* store the configurations */
   		try {   			
   			
        	properties.store(new FileOutputStream("teacher.properties"), "Teacher configuration");        	
        	
   		} catch (IOException ex) {
        	ExceptionDialogBox.displayExceptionDialog(this,ex, "Error saving the configurations in teacher.properties");
        }
   		
   		/* store the commands */
   		try {
   			
   			Properties storedCommands = new Properties();   			
   			
        	Iterator i = commands.iterator();
        	
        	while (i.hasNext()) {
        		Command cmd = (Command) i.next();
        		        		
        		String cmdString = cmd.getDescription() + "#" + cmd.getCommand();
        		
        		storedCommands.setProperty(cmd.getName(), cmdString);
        	}
        	
        	storedCommands.store(new FileOutputStream("commands.properties"), "Teacher commands");
        	
   		} catch (Exception ex) {
   			ExceptionDialogBox.displayExceptionDialog(this,ex, "Error saving the commands in commands.properties");
        }
   		
   		
	}
	
	private JComboBox commandList = null;
	private JButton executeCommandButton = null;
	private ArrayList commands;
	private JButton configureButton = null;
	
	private void setupCommandList() {
		
		try {
   			Properties commandfile = new Properties();
   			commandfile.load(new FileInputStream("commands.properties"));
        	Iterator i = commandfile.entrySet().iterator();
        	
        	while (i.hasNext()) {
        		Map.Entry entry = (Map.Entry) i.next();
        		StringTokenizer st = new StringTokenizer((String) entry.getValue(), "#");
        		commands.add(new Command((String) entry.getKey(), st.nextToken(), st.nextToken()));
        		System.out.println("added" +commands.get(commands.size()-1));
        	}
        	
        	
   		} catch (Exception ex) {
   			ExceptionDialogBox.displayExceptionDialog(this,ex, "Error while loading the command list from commands.properties");
        }	
   		
   		
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JToolBar getJPanel() {
		if (buttonPanel == null) {
			lblFullscreen = new JLabel();
			lblFullscreen.setText("Fullscreen");
			buttonPanel = new JToolBar();
			buttonPanel.setFloatable(false);
			buttonPanel.setMargin(new Insets(5,5,5,5));
			buttonPanel.setRollover(true);
			buttonPanel.setName("buttonPanel");
			buttonPanel.add(getJButton(), null);
			buttonPanel.add(getJButton2(), null);
			buttonPanel.add(getJCheckBox(), null);
			buttonPanel.add(lblFullscreen, null);
			buttonPanel.addSeparator();
			buttonPanel.add(getLockScreenButton(), null);
			buttonPanel.addSeparator();
			buttonPanel.add(getJButton3(), null);
			buttonPanel.add(getJButton4(), null);
			buttonPanel.addSeparator();
			buttonPanel.add(getJButton5(), null);
			buttonPanel.add(getCommandList(), null);
			buttonPanel.addSeparator();
			buttonPanel.add(getJButton6(), null);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (startDemoButton == null) {
			startDemoButton = new JButton();
			startDemoButton.setText("Start demo");
			startDemoButton.setActionCommand("Start demo on all");
			startDemoButton.setName("startDemoButton");
			startDemoButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {						
					hub.sendStartDemoToAll(chkFullscreen.isSelected());
				}
			}); 
		} 
		return startDemoButton;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton2() {
		if (stopDemoButton == null) {
			stopDemoButton = new JButton();
			stopDemoButton.setName("stopDemoButton");
			stopDemoButton.setActionCommand("Stop demo on all");
			stopDemoButton.setText("Stop demo");
			stopDemoButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					hub.sendStopDemoToAll();
				}
			});
		}
		return stopDemoButton;
	}

	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBox() {
		if (chkFullscreen == null) {
			chkFullscreen = new JCheckBox();
			chkFullscreen.setName("chkFullscreen");
		}
		return chkFullscreen;
	}

	/**
	 * This method initializes jToggleButton	
	 * 	
	 * @return javax.swing.JToggleButton	
	 */
	private JToggleButton getLockScreenButton() {
		if (lockScreenButton == null) {
			lockScreenButton = new JToggleButton();
			lockScreenButton.setText("Lock");
			lockScreenButton.setName("lockScreenButton");
			lockScreenButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					hub.sendLockScreenToAll(lockScreenButton.isSelected());
				}
			});
		}
		return lockScreenButton;
	}

	/**
	 * This method initializes jDesktopPane	
	 * 	
	 * @return javax.swing.JDesktopPane	
	 */
	private JDesktopPane getJDesktopPane() {
		if (jDesktopPane == null) {
			jDesktopPane = new JDesktopPane();
		}
		return jDesktopPane;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton3() {
		if (saveConfigButton == null) {
			saveConfigButton = new JButton();
			saveConfigButton.setText("Save layout");
			saveConfigButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					hub.savePositions();
					setProperty("height", "" + getHeight());
					setProperty("width", "" + getWidth());
				}
			});
		}
		return saveConfigButton;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton4() {
		if (ReloadLayoutButton == null) {
			ReloadLayoutButton = new JButton();
			ReloadLayoutButton.setText("Reload layout");
			ReloadLayoutButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					hub.relayoutStudents();
				}
			});
		}
		return ReloadLayoutButton;
	}

	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getCommandList() {
		if (commandList == null) {
			commandList = new JComboBox();
			
			setupCommandList();
			commandList.setModel(new DefaultComboBoxModel(commands.toArray()));
		}
		return commandList;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton5() {
		if (executeCommandButton == null) {
			executeCommandButton = new JButton();
			executeCommandButton.setText("Do");
			executeCommandButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					hub.executeOnAll(((Command) commandList.getSelectedItem()));
				}
			});
		}
		return executeCommandButton;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton6() {
		if (configureButton == null) {
			configureButton = new JButton();
			configureButton.setText("Setup");
			configureButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					JTeacherConfigurationFrame conf = new JTeacherConfigurationFrame(JTeacherFrame.this);
					conf.setVisible(true);
				}
			});
		}
		return configureButton;
	}

	public void addNewStudent(JStudentThumbnail s) {
		s.setBounds(hub.getPreferredPosition(s.getStudent().getHost()));
		jDesktopPane.add(s);
	}
	

	public boolean isFullscreenSelected() {
		return getJCheckBox().isSelected();
	}

	public String getProperty(String key, String defaultValue) {
		return properties.getProperty(key, defaultValue);
	}

	public String getProperty(String key) {
		return properties.getProperty(key);
	}

	public Object setProperty(String key, String value) {
		return properties.setProperty(key, value);
	}



	
	public String getFullname(String login) {
		if (login == null) return null;
		if (fullnames.containsKey(login)) {
			return (String) fullnames.get(login);
		} else {
			String fullname = Utility.executeCommand(properties.getProperty("finger").replaceAll("%user", login));
			fullnames.put(login,fullname);
			return fullname;
		}
	}
	
	
	public Command getSelectedCommand() {
		return (Command) commandList.getSelectedItem();
	}
	
	public ArrayList getAllCommands() {
		return commands;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
